from typing import List, Optional, Any

from numpy import ndarray, _OrderCF
from numpy.typing import ArrayLike, DTypeLike

__all__: List[str]

def fliplr(m): ...
def flipud(m): ...

def eye(
    N: int,
    M: Optional[int] = ...,
    k: int = ...,
    dtype: DTypeLike = ...,
    order: _OrderCF = ...,
    *,
    like: Optional[ArrayLike] = ...
) -> ndarray[Any, Any]: ...

def diag(v, k=...): ...
def diagflat(v, k=...): ...
def tri(N, M=..., k=..., dtype = ..., *, like=...): ...
def tril(m, k=...): ...
def triu(m, k=...): ...
def vander(x, N=..., increasing=...): ...
def histogram2d(x, y, bins=..., range=..., normed=..., weights=..., density=...): ...
def mask_indices(n, mask_func, k=...): ...
def tril_indices(n, k=..., m=...): ...
def tril_indices_from(arr, k=...): ...
def triu_indices(n, k=..., m=...): ...
def triu_indices_from(arr, k=...): ...
