from typing import Any, List

from numpy import (
    chararray as chararray,
)

__all__: List[str]

def equal(x1, x2): ...
def not_equal(x1, x2): ...
def greater_equal(x1, x2): ...
def less_equal(x1, x2): ...
def greater(x1, x2): ...
def less(x1, x2): ...
def str_len(a): ...
def add(x1, x2): ...
def multiply(a, i): ...
def mod(a, values): ...
def capitalize(a): ...
def center(a, width, fillchar=...): ...
def count(a, sub, start=..., end=...): ...
def decode(a, encoding=..., errors=...): ...
def encode(a, encoding=..., errors=...): ...
def endswith(a, suffix, start=..., end=...): ...
def expandtabs(a, tabsize=...): ...
def find(a, sub, start=..., end=...): ...
def index(a, sub, start=..., end=...): ...
def isalnum(a): ...
def isalpha(a): ...
def isdigit(a): ...
def islower(a): ...
def isspace(a): ...
def istitle(a): ...
def isupper(a): ...
def join(sep, seq): ...
def ljust(a, width, fillchar=...): ...
def lower(a): ...
def lstrip(a, chars=...): ...
def partition(a, sep): ...
def replace(a, old, new, count=...): ...
def rfind(a, sub, start=..., end=...): ...
def rindex(a, sub, start=..., end=...): ...
def rjust(a, width, fillchar=...): ...
def rpartition(a, sep): ...
def rsplit(a, sep=..., maxsplit=...): ...
def rstrip(a, chars=...): ...
def split(a, sep=..., maxsplit=...): ...
def splitlines(a, keepends=...): ...
def startswith(a, prefix, start=..., end=...): ...
def strip(a, chars=...): ...
def swapcase(a): ...
def title(a): ...
def translate(a, table, deletechars=...): ...
def upper(a): ...
def zfill(a, width): ...
def isnumeric(a): ...
def isdecimal(a): ...
def array(obj, itemsize=..., copy=..., unicode=..., order=...): ...
def asarray(obj, itemsize=..., unicode=..., order=...): ...
